package main

import (
	"bytes"
	"fmt"
	"go/format"
	"log"
	"net"
	"os"
	"sort"
)

func main() {
	var (
		result []struct {
			Name string
			NS   []string
		}
		code bytes.Buffer
		err  error
	)

	// get nameservers for domains
	for _, target := range []struct {
		Domain  string
		VarName string
	}{
		{
			Domain:  "kit.edu.",
			VarName: "KITNameserver",
		},
		{
			Domain:  "uni-beispiel.de.",
			VarName: "UniBeispielNameserver",
		},
	} {
		ns, err := net.LookupNS(target.Domain)
		if err != nil {
			log.Panic(err)
		}
		var nsList []string
		for _, ns := range ns {
			nsList = append(nsList, ns.Host)
		}
		// sort list of nameservers
		sort.Strings(nsList)
		result = append(result, struct {
			Name string
			NS   []string
		}{
			Name: target.VarName,
			NS:   nsList,
		})
	}

	// sort result via variable name
	sort.Slice(result, func(i, j int) bool { return result[i].Name < result[j].Name })
	// write lists as go source
	code.WriteString(
		`// AUTOGENERATED FILE. DO NOT EDIT MANUALLY.
package acme4netvs

var (
`)
	for _, v := range result {
		code.WriteString(fmt.Sprintf("%s = %#v\n", v.Name, v.NS))
	}
	code.WriteString(`)`)

	// open output file
	f, err := os.OpenFile("zzz_autogenerated_nslists.go", os.O_CREATE|os.O_TRUNC|os.O_WRONLY, 0644)
	if err != nil {
		log.Panic(err)
	}

	// Ignore error on close, this is a readonly file anyway
	defer func(f *os.File) {
		_ = f.Close()
	}(f)

	// format code
	formattedCode, err := format.Source(code.Bytes())
	if err != nil {
		_, err = f.Write(code.Bytes())
	} else {
		_, err = f.Write(formattedCode)
	}
	if err != nil {
		panic(err)
	}
	err = f.Close()
	if err != nil {
		panic(err)
	}
}
