//go:generate net-api-generator golang-structs --package acme4netvs -o zzz_autogenerated_netvs_types.go
package acme4netvs

import (
	"bytes"
	"crypto/tls"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"net"
	"net/http"
	"time"

	"github.com/HereMobilityDevelopers/mediary"
)

const (
	DefaultAPIVersion = "4.1"
	URITemplate       = `https://%s/api/%s/wapi/transaction/execute`
	BASEURITEST       = "api.netdb-test.scc.kit.edu"
	BASEURIPROD       = "api.netdb.scc.kit.edu"
	BASEURIDEVEL      = "api.netdb-devel.scc.kit.edu"
)

// GetSessionToken returns a string containing a session token
type GetSessionToken func() string

type NETVSConfig struct {
	BaseURI      string
	APIVersion   string
	SessionToken GetSessionToken
}

type NETVSClient struct {
	NETVSConfig
	Client *http.Client
}

type TAResult []interface{}

func (r TAResult) NumResults() int {
	return len(r)
}

func (r TAResult) IsError() bool {
	return true
}

type AuthorizationError error

type InvalidRequest error

type InternalServerError error

type OtherUnexpectedResponseError error

// NewNETVSClient creates a new NETVSClient that automatically adds authentication headers
func NewNETVSClient(config *NETVSConfig) *NETVSClient {
	client := mediary.Init().
		WithPreconfiguredClient(
			&http.Client{
				Transport: &http.Transport{
					Proxy: http.ProxyFromEnvironment,
					DialContext: (&net.Dialer{
						Timeout:   30 * time.Second,
						KeepAlive: 30 * time.Second,
					}).DialContext,
					ForceAttemptHTTP2:   true,
					IdleConnTimeout:     90 * time.Second,
					TLSHandshakeTimeout: 10 * time.Second,
					// this is currently required
					TLSClientConfig: &tls.Config{
						MinVersion:         tls.VersionTLS12,
						InsecureSkipVerify: false,
						Renegotiation:      tls.RenegotiateOnceAsClient,
					},
				},
				CheckRedirect: nil,
				Timeout:       60 * time.Second,
			}).
		AddInterceptors(func(req *http.Request, handler mediary.Handler) (*http.Response, error) {
			req.Header.Add("Content-Type", "application/json")
			req.Header.Add("Authorization", "Bearer "+config.SessionToken())
			return handler(req)
		}).
		Build()

	return &NETVSClient{
		NETVSConfig: *config,
		Client:      client,
	}
}

func ReaderToString(r io.Reader) string {
	var buf bytes.Buffer
	_, _ = buf.ReadFrom(r)
	return buf.String()
}

func (c *NETVSClient) ExecuteTA(t Transaction) (TAResult, error) {
	var response TAResult

	// return empty result if no transactions are specified
	if t.NumEntries() == 0 {
		return response, nil
	}

	// create HTTP request
	req, err := http.NewRequest(
		"POST",
		fmt.Sprintf(URITemplate, c.NETVSConfig.BaseURI, c.NETVSConfig.APIVersion),
		t.AsRequestBody())
	if err != nil {
		return response, err
	}

	// send request
	resp, err := c.Client.Do(req)
	if err != nil {
		return response, err
	}
	defer func() {
		// Drain and close the body to let the Transport reuse the connection
		// see https://github.com/google/go-github/pull/317 for details
		_, _ = io.Copy(io.Discard, resp.Body)
		_ = resp.Body.Close()
	}()

	// handle errors based on HTTP return code
	switch resp.StatusCode {
	case 200: // success
		break
	case 400: // bad request
		return response, InvalidRequest(fmt.Errorf(ReaderToString(resp.Body)))
	case 401: // Unauthorized
		return response, AuthorizationError(fmt.Errorf(ReaderToString(resp.Body)))
	case 500: // internal server error
		return response, InternalServerError(fmt.Errorf(ReaderToString(resp.Body)))
	default: // everything else
		return response, OtherUnexpectedResponseError(errors.New(resp.Status))
	}

	// copy response body to buffer
	r, err := io.ReadAll(resp.Body)
	if err != nil {
		return response, err
	}

	// convert response to array of array
	err = json.Unmarshal(r, &response)
	if err != nil {
		return response, fmt.Errorf("unable to decode result as JSON: %w", err)
	}

	return response, nil
}

type Transaction interface {
	// AsRequestBody returns the complete transaction as a string for the request body
	AsRequestBody() io.Reader
	// AddEntries appends transaction entries
	AddEntries(request ...TransactionEntry)
	// NumEntries returns the number of entries
	NumEntries() int
}

type TransactionEntry interface {
	// AsPart returns a single request as json object for the transaction body
	AsPart() string
}

// TransactionList holds the list of statements in a transaction.
// It implements the Transaction interface.
type TransactionList []TransactionEntry

// NewTARequestBody creates an empty transactionList
func NewTARequestBody() *TransactionList {
	return &TransactionList{}
}

func (body *TransactionList) AsRequestBody() io.Reader {
	var bodyreader = bytes.Buffer{}

	jsonbody, err := json.Marshal(body)
	if err != nil {
		return &bodyreader
	}
	bodyreader.Write(jsonbody)

	return &bodyreader
}

func (body *TransactionList) AddEntries(request ...TransactionEntry) {
	*body = append(*body, request...)
}

func (body *TransactionList) NumEntries() int {
	return len(*body)
}

// TAStatement represents a single transaction statement
type TAStatement struct {
	Name string             `json:"name"`
	Old  TAStatementDataMap `json:"old,omitempty"`
	New  TAStatementDataMap `json:"new,omitempty"`
}

func (taStatement TAStatement) AsPart() string {
	jsonobj, err := json.Marshal(taStatement)
	if err != nil {
		return `{ name: "invalid.transaction" }`
	}
	return string(jsonobj)
}

// TAStatementDataMap is a typed map for members Old and New of TAStatement
type TAStatementDataMap map[string]interface{}
